import React from 'react';
import { ReadabilityIcon } from '../Icons';

const ReadabilitySection = ({
  rawReadabilityLevel,
  readability,
  toggleSection,
  expandedSections
}) => {
  // Readability level processing
  const readabilityLevel = rawReadabilityLevel || 'na';

  // Ensure readability percentage is properly calculated and capped at 100%
  const rawReadability = readability || 0.33;
  // Make sure we're using the actual number, not multiplying it again if it's already a percentage
  const normalizedReadability = rawReadability > 1 ? rawReadability / 100 : rawReadability;
  const readabilityPercentage = Math.min(100, Math.round(normalizedReadability * 100));

  // Map readability percentage to appropriate grade level based on the table
  let gradeLevel = '';
  let readabilityDescription = '';

  switch (readabilityPercentage) {
    case readabilityPercentage >= 90:
      gradeLevel = '5th grade';
      readabilityDescription = 'Very easy to read. Easily understood by students who are generally around 11 years old.';
      break;
    case readabilityPercentage >= 80:
      gradeLevel = '6th grade';
      readabilityDescription = 'Easy to read. Conversational English for consumers.';
      break;
    case readabilityPercentage >= 70:
      gradeLevel = '7th grade';
      readabilityDescription = 'Fairly easy to read.';
      break;
    case readabilityPercentage >= 60:
      gradeLevel = '8th & 9th grade';
      readabilityDescription = 'Plain English. Easily understood by students in these grades, who are generally 13-15 years old.';
      break;
    case readabilityPercentage >= 50:
      gradeLevel = '10th to 12th grade';
      readabilityDescription = 'Fairly difficult to read, students in these grades are generally 15-18 years old.';
      break;
    case readabilityPercentage >= 30:
      gradeLevel = 'College';
      readabilityDescription = 'Difficult to read.';
      break;
    case readabilityPercentage >= 10:
      gradeLevel = 'College graduate';
      readabilityDescription = 'Very difficult to read. Best understood by university graduates.';
      break;
    default:
      gradeLevel = 'Professional';
      readabilityDescription = 'Extremely difficult to read. Best understood by university graduates.';
      break;
  }

  const readabilityScore = readabilityLevel !== 'na'
      ? `${gradeLevel}: ${readabilityPercentage}%`
      : 'N/A';

  return (
    <div className="ao-category-section">
      <div
        className="ao-category-header"
        onClick={() => toggleSection('readability')}
      >
        <div className="ao-category-title">
          <ReadabilityIcon />
          <h3 className="ao-category-name">Readability</h3>
        </div>

        <div className="ao-category-count">
          <span style={{ fontSize: '16px', fontWeight: '500' }}>{readabilityScore}</span>
          <svg
            className={`ao-category-arrow ${expandedSections.readability ? 'ao-category-arrow--expanded' : ''}`}
            viewBox="0 0 24 24"
            fill="#333"
          >
            <path d="M7 10L12 15L17 10H7Z"/>
          </svg>
        </div>
      </div>

      {expandedSections.readability && (
        <div className="ao-category-content">
          <div className="ao-readability-info">
            <strong className="ao-readability-grade">{gradeLevel}:</strong>
            <span className="ao-readability-description">{readabilityDescription}</span>
          </div>

          <p className="ao-readability-explanation">
            Readability refers to how easily a reader can understand written text.
            The score uses the Flesch-Kincaid grade level, indicating the US school
            grade needed to comprehend the text.
          </p>
          <p className="ao-readability-explanation">
            A lower grade level (e.g., 8th grade) means the content is more accessible
            to a wider audience. Aim for 8th-10th grade for most web content.
          </p>
        </div>
      )}
    </div>
  );
};

export default ReadabilitySection;
