

export class HtmlCssCollector {
  constructor(isDebugMode = false) {
    this.isDebugMode = isDebugMode;
  }

  /**
   * Collects HTML and CSS from various sources in priority order
   * @param {Object} layout - Redux layout state
   * @returns {Promise<{pageHtml: string, pageCss: string}>}
   */
  async collectPageContent(layout) {
    let pageHtml = '';
    let pageCss = '';

    // Try to get content from Experience Builder iframe first
    const iframeContent = await this.getContentFromIframe();
    if (iframeContent.html) {
      pageHtml = iframeContent.html;
      pageCss = iframeContent.css;
    }

    // Fallback to current document if iframe content is insufficient
    if (!pageHtml || pageHtml.length < 1000) {
      const documentContent = this.getContentFromDocument();
      pageHtml = documentContent.html;
      if (!pageCss) {
        pageCss = documentContent.css;
      }
    }

    // Last resort: Use layout data
    if (!pageHtml && layout) {
      pageHtml = this.generateHtmlFromLayout(layout);
    }

    // Ensure we have some CSS if not collected yet
    if (!pageCss) {
      pageCss = this.collectMainDocumentCss();
    }

    return { pageHtml, pageCss };
  }

  /**
   * Attempts to extract content from Experience Builder iframe
   * @returns {Promise<{html: string, css: string}>}
   */
  async getContentFromIframe() {
    try {
      const xbIframe = document.querySelector('[data-xb-preview="lg"]');

      if (!xbIframe || !xbIframe.contentDocument) {
        if (this.isDebugMode) {
          console.warn('Experience Builder iframe not found or contentDocument not accessible');
        }
        return { html: '', css: '' };
      }

      // Get the full HTML content from the iframe
      let html = xbIframe.contentDocument.documentElement.outerHTML;

      // Ensure proper DOCTYPE
      if (!html.includes('<!DOCTYPE')) {
        html = '<!DOCTYPE html>\n' + html;
      }

      // Collect CSS from iframe
      const css = this.collectIframeCss(xbIframe);

      return { html, css };
    } catch (error) {
      if (this.isDebugMode) {
        console.warn('Error accessing Experience Builder iframe content:', error.message);
      }
      return { html: '', css: '' };
    }
  }

  /**
   * Collects CSS from iframe stylesheets
   * @param {HTMLIFrameElement} iframe
   * @returns {string}
   */
  collectIframeCss(iframe) {
    const iframeCss = [];

    try {
      const sheets = iframe.contentDocument.styleSheets;

      for (let i = 0; i < sheets.length; i++) {
        try {
          const rules = sheets[i].cssRules;
          for (let j = 0; j < rules.length; j++) {
            iframeCss.push(rules[j].cssText);
          }
        } catch (corsError) {
          if (this.isDebugMode) {
            console.warn('CORS issue accessing stylesheet:', corsError);
          }

          // Try to add stylesheet link if available
          if (sheets[i].href) {
            this.addStylesheetLink(sheets[i].href, iframe.contentDocument);
          }
        }
      }
    } catch (cssError) {
      if (this.isDebugMode) {
        console.warn('Error collecting CSS from iframe:', cssError);
      }
    }

    return iframeCss.join('\n');
  }

  /**
   * Adds a stylesheet link to the document head
   * @param {string} href - Stylesheet URL
   * @param {Document} targetDocument - Document to add link to
   */
  addStylesheetLink(href, targetDocument) {
    try {
      const linkElem = targetDocument.createElement('link');
      linkElem.setAttribute('rel', 'stylesheet');
      linkElem.setAttribute('href', href);

      const head = targetDocument.head;
      if (head && !head.querySelector(`link[href="${href}"]`)) {
        head.appendChild(linkElem);
      }
    } catch (linkError) {
      if (this.isDebugMode) {
        console.warn('Error adding stylesheet link:', linkError);
      }
    }
  }

  /**
   * Gets content from the current document
   * @returns {{html: string, css: string}}
   */
  getContentFromDocument() {
    try {
      const html = document.documentElement.outerHTML;
      const css = this.collectMainDocumentCss();

      return { html, css };
    } catch (error) {
      if (this.isDebugMode) {
        console.warn('Error getting document content:', error);
      }
      return { html: '', css: '' };
    }
  }

  /**
   * Generates basic HTML from layout data as fallback
   * @param {Object} layout - Redux layout state
   * @returns {string}
   */
  generateHtmlFromLayout(layout) {
    if (this.isDebugMode) {
      console.warn('Using layout data as fallback - this may produce limited results');
    }

    return `<!DOCTYPE html>
<html>
  <head>
    <title>Experience Builder Preview</title>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
  </head>
  <body>
    <div class="layout-container">
      ${JSON.stringify(layout, null, 2)}
    </div>
  </body>
</html>`;
  }

  /**
   * Collects CSS from main document stylesheets
   * @returns {string}
   */
  collectMainDocumentCss() {
    const mainDocumentCss = [];

    try {
      const sheets = document.styleSheets;

      for (let i = 0; i < sheets.length; i++) {
        // Skip admin-specific stylesheets
        if (this.isAdminStylesheet(sheets[i])) {
          continue;
        }

        try {
          const rules = sheets[i].cssRules;
          for (let j = 0; j < rules.length; j++) {
            mainDocumentCss.push(rules[j].cssText);
          }
        } catch (corsError) {
          if (this.isDebugMode) {
            console.warn('CORS issue accessing stylesheet:', corsError);
          }
        }
      }
    } catch (cssError) {
      if (this.isDebugMode) {
        console.warn('Error collecting CSS from main document:', cssError);
      }
    }

    return mainDocumentCss.join('\n');
  }

  /**
   * Checks if a stylesheet is admin-related and should be skipped
   * @param {CSSStyleSheet} sheet
   * @returns {boolean}
   */
  isAdminStylesheet(sheet) {
    if (!sheet.href) return false;

    return sheet.href.includes('admin') ||
           sheet.href.includes('toolbar') ||
           sheet.href.includes('seven');
  }

}
