import { debugError } from '../utils/constants';

export class PollingService {
  constructor(apiService, isDebugMode = false) {
    this.apiService = apiService;
    this.isDebugMode = isDebugMode;
  }

  /**
   * Polls scan results until completion or failure
   * @param {string} scanId - The scan ID to poll
   * @param {Object} settings - API settings
   * @param {number} progressInterval - Progress animation interval ID
   * @param {Function} setScanProgress - Progress update callback
   * @param {Function} setScanResults - Results update callback
   * @param {Function} setCurrentView - View update callback
   */
  async pollScanResults(scanId, settings, progressInterval, setScanProgress, setScanResults, setCurrentView) {
    try {
      if (!scanId) {
        throw new Error('Invalid scan ID');
      }

      // Check scan status
      const response = await fetch(`/acquia-optimize/api/quick-scan/${scanId}`, {
        method: 'GET',
        headers: {
          'Accept': 'application/json',
        }
      });

      if (response.ok) {
        const data = await response.json();
        const status = data.status;

        this.handleScanStatus(
          status,
          data,
          scanId,
          settings,
          progressInterval,
          setScanProgress,
          setScanResults,
          setCurrentView
        );
      } else {
        await this.handlePollingError(
          response,
          progressInterval,
          setScanProgress,
          setScanResults,
          setCurrentView
        );
      }
    } catch (error) {
      this.handlePollingException(
        error,
        progressInterval,
        setScanProgress,
        setScanResults,
        setCurrentView
      );
    }
  }

  /**
   * Handles different scan status values
   * @param {string} status - Current scan status
   * @param {Object} data - Full response data
   * @param {string} scanId - Scan ID
   * @param {Object} settings - API settings
   * @param {number} progressInterval - Progress animation interval
   * @param {Function} setScanProgress - Progress update callback
   * @param {Function} setScanResults - Results update callback
   * @param {Function} setCurrentView - View update callback
   */
  handleScanStatus(status, data, scanId, settings, progressInterval, setScanProgress, setScanResults, setCurrentView) {
    const inProgressStatuses = ['scanning', 'queued', 'processing', 'data_protection'];
    const completedStatuses = ['completed', 'processed'];
    const failedStatuses = ['error', 'failed'];

    if (inProgressStatuses.includes(status)) {
      // Continue polling
      setTimeout(() => {
        this.pollScanResults(scanId, settings, progressInterval, setScanProgress, setScanResults, setCurrentView);
      }, 2000);
    } else if (completedStatuses.includes(status)) {
      // Scan complete
      this.handleScanCompletion(data, progressInterval, setScanProgress, setScanResults, setCurrentView);
    } else if (failedStatuses.includes(status) || data.error) {
      // Scan failed
      this.handleScanFailure(data, progressInterval, setScanProgress, setScanResults, setCurrentView);
    } else {
      // Unknown status
      this.handleUnknownStatus(status, data, scanId, settings, progressInterval, setScanProgress, setScanResults, setCurrentView);
    }
  }

  /**
   * Handles successful scan completion
   */
  handleScanCompletion(data, progressInterval, setScanProgress, setScanResults, setCurrentView) {
    clearInterval(progressInterval);
    setScanProgress(100);
    setScanResults(data.scan_result || data);
    setTimeout(() => {
      setCurrentView('results');
    }, 500);
  }

  /**
   * Handles scan failure
   */
  handleScanFailure(data, progressInterval, setScanProgress, setScanResults, setCurrentView) {
    clearInterval(progressInterval);
    setScanProgress(100);
    setScanResults({
      status: 'failed',
      error: true,
      message: data.failure_reason || data.message || data.error_message || 'Scan failed'
    });
    setTimeout(() => {
      setCurrentView('results');
    }, 500);
  }

  /**
   * Handles unknown scan status
   */
  handleUnknownStatus(status, data, scanId, settings, progressInterval, setScanProgress, setScanResults, setCurrentView) {
    // If the status contains "process", continue polling
    if (typeof status === 'string' && status.toLowerCase().includes('process')) {
      setTimeout(() => {
        this.pollScanResults(scanId, settings, progressInterval, setScanProgress, setScanResults, setCurrentView);
      }, 2000);
    } else {
      // Stop polling and show results with warning
      clearInterval(progressInterval);
      setScanProgress(100);

      const results = data.scan_result ?
        data.scan_result :
        { ...data, warning: `Unknown scan status: ${status}` };

      setScanResults(results);
      setTimeout(() => {
        setCurrentView('results');
      }, 500);

    }
  }

  /**
   * Handles polling API errors
   */
  async handlePollingError(response, progressInterval, setScanProgress, setScanResults, setCurrentView) {
    let errorData;
    let errorMessage;

    try {
      errorData = await response.json();
      debugError(this.isDebugMode, 'Status check API error (JSON):', errorData);
      errorMessage = errorData.failure_reason ||
                    errorData.message ||
                    errorData.error_message ||
                    `Failed to check scan status: ${response.status} ${response.statusText}`;
    } catch (e) {
      errorData = await response.text();
      debugError(this.isDebugMode, 'Status check API error (Text):', errorData);
      errorMessage = `Failed to check scan status: ${response.status} ${response.statusText}`;
    }

    clearInterval(progressInterval);
    setScanProgress(100);
    setScanResults({
      status: 'failed',
      error: true,
      failure_reason: errorData?.failure_reason,
      message: errorMessage
    });
    setTimeout(() => {
      setCurrentView('results');
    }, 500);
  }

  /**
   * Handles polling exceptions
   */
  handlePollingException(error, progressInterval, setScanProgress, setScanResults, setCurrentView) {
    debugError(this.isDebugMode, 'Error polling scan results:', error);
    clearInterval(progressInterval);
    setScanProgress(100);
    setScanResults({
      status: 'failed',
      error: true,
      message: error.message || 'Failed to check scan status'
    });
    setTimeout(() => {
      setCurrentView('results');
    }, 500);
  }

}
